/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <QMap>
#include <QtGui/QMainWindow>
#include <QTimer>
#include <QTime>

#include "defines.h"
#include "Parser.h"

class QAction;
class QActionGroup;
class QGraphicsView;
class QGraphicsScene;
class QGraphicsPixmapItem;
class QCloseEvent;
class QMenu;
class QModelIndex;
class EquationPixmapItem;
class FrameMathTab;
class LatexEditor;
class WidgetFind;
class WidgetToolMenu;
class DialogPreferences;
class DialogReplace;
class DialogAbout;
class QPushButton;
class QToolButton;
class QSignalMapper;

namespace Ui
{
    class MainWindowClass;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void updateEquation(); // updates tex, png, and scene
    void updateImage(); // only updates the image file DVI2PNG
    void updateDisplay(); // updates scene

    void onEquationChanged(bool redraw=true);

    void onFindTextChanged(const QString &exp);

    void showFindDialog();
    void showReplaceDialog();

    void setDirty() { isDirty = true; }

    void insertSymbol(QAction *action=0);

private slots:

    void editorChanged();
    bool exportToDVI();
    bool exportToGIF();
    bool exportToPDF();
    bool exportToPS();
    bool exportToROOT();
    bool exportToTeX();

    void newEquation();
    void onAbout();
    void onDeleteSelection(); // delete selected text from editor
    bool onPreferences(); // show preferences dialog
    void open();
    void openRecentFile();
    bool save();
    bool saveAs();

    void writeSettings();
    void readSettings();

signals:
    void zoomChanged();

    // emit this signal whenever the equation has changed
    //  isDirty is true when the display needs redrawing
    void equationChanged(bool isDirty);


protected:
    void clearCurrentHighlight();
    void setActions();
    void setSymbolsPanel();
    void setMathPanelTabsActions();

    void setSignals();
    void setCurrentFile(const QString &fileName);

    bool wishToSave(); // if user wishes to save modifications
    void loadEquation(const QString &fileName);
    bool saveEquation(QString &fileName);
    QString saveTo(const QString &filter);
    void writePNGMetaData(const QString &pngFile);

    void loadTexTemplate();

    void setupParser();
    void closeEvent(QCloseEvent *);


   /*
        Adds ToolButtons with Actions conforming the file found by file name at eqImage
    to the widget
    */
    void addButtonToMathTabs(const QString &eqImage, QWidget *widget);
    void addButtonToMathTabs(const QStringList &list, QWidget *widget);

    void updateUI(); // update Main Window UI according to settings
    void updateRecentFileActions();
    QString strippedName(const QString &fullFileName);

private:
    bool runCommand(const QString &commandStr);

    /*  MainWindow widgets */
    Ui::MainWindowClass *ui;

    EquationPixmapItem *equation;
    DialogReplace *replaceDialog;
    DialogPreferences *preferencesDialog;
    DialogAbout *aboutDialog;
    LatexEditor *editor;
    Parser parser;
    WidgetFind *findWidget;
    FrameMathTab *templateTab;

    QAction *recentFileActions[MAX_RECENT_FILES];
    QGraphicsScene *scene;
    QSignalMapper *signalMapper;


    /* Symbols Menus */
    WidgetToolMenu *toolDeco;
    WidgetToolMenu *toolFunc;
    WidgetToolMenu *toolDelim;
    WidgetToolMenu *toolAccents;
    WidgetToolMenu *toolOps;
    WidgetToolMenu *toolRelations;
    WidgetToolMenu *toolMisc;
    WidgetToolMenu *toolSets;
    WidgetToolMenu *toolGreek;
    WidgetToolMenu *toolGreekBig;
    WidgetToolMenu *toolSums;
    WidgetToolMenu *toolIntegrals;
    WidgetToolMenu *toolProd;
    WidgetToolMenu *toolArrows;
    WidgetToolMenu *toolSpacing;

    // data
    QString tempFileName; // path to output temp file
    QString curFile; // current file
    QString texTemplateContent; // content of the tex template

    // flags
    bool isDirty; // wheater to update equation
    bool highlightAll;
    bool renderAuto;


    int zoomFactor;
    int nUpdates;

};

#endif // MAINWINDOW_H
